SystemOrganization addCategory: #PromisePipelining!
SystemOrganization addCategory: #'PromisePipelining-Tests'!
SystemOrganization addCategory: #'PromisePipelining-Visualisation'!
SystemOrganization addCategory: #'PromisePipelining-Signals'!

!Exception methodsFor: '*PromisePipelining' stamp: 'WS 1/12/2012 13:37'!
asPromiseReference
	^ PPExceptionReference for: self! !

Promise subclass: #PPPromise
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining'!

PPPromise class
	instanceVariableNames: 'creationSignal connectSignal'!

PPPromise class
	instanceVariableNames: 'creationSignal connectSignal'!

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:31'!
connectSignal
	^ connectSignal ifNil: [ connectSignal := PPSignal new ]! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:30'!
creationSignal
	^ creationSignal ifNil: [ creationSignal := PPSignal new ]! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:31'!
disconnectConnect: id
	self connectSignal disconnect: id! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:11'!
disconnectCreation: id
	self creationSignal disconnect: id! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 12:50'!
new
	| promise |
	promise := super new.
	self creationSignal emitWith: promise.
	^ promise! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:31'!
onConnect: aBlock
	^ self connectSignal connect: aBlock! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:11'!
onCreation: aBlock
	^ self creationSignal connect: aBlock! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:22'!
promiseFrom: aReferencedObject forMessage: aMessage onResolveSendTo: aBlock
	| promise |
	promise := self new.
	
	self connectSignal emitWith: aReferencedObject with: promise with: aMessage.
	
	^ promise initializeFrom: aReferencedObject forMessage: aMessage onResolveSendTo: aBlock! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:45'!
asFuture
	^ PPFuture for: self! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:02'!
checkForExceptionOrReturn: anObject
	^ anObject isExceptionReference
		ifTrue: [ anObject raise ]
		ifFalse: [ anObject ]! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:00'!
directValue
	^ self checkForExceptionOrReturn: super value! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:38'!
doesNotUnderstand: aMessage
	^ PPPromise promiseFrom: self forMessage: aMessage onResolveSendTo: [ self directValue ]! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:22'!
initializeFrom: aReferencedObject forMessage: aMessage onResolveSendTo: aBlock
	| lastBlock |
	lastBlock := [ :arguments |
		self resolveWith: ([ aBlock value perform: aMessage selector withArguments: arguments ] on: Exception do: [ :exception | exception asPromiseReference ])
	].
	aMessage arguments withIndexDo: [ :argument :index |
		lastBlock := self prepareBlock: lastBlock forArgument: argument withIndex: index.
	].
	
	aReferencedObject whenResolved: [ lastBlock value: aMessage arguments ].
	
	^ self asFuture! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:02'!
isBroken
	^ super value isExceptionReference! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:32'!
isPromise
	^ true! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:41'!
prepareBlock: lastBlock forArgument: argument withIndex: index
	^ argument isPromise ifTrue: [
		[ :arguments |
			argument whenResolved: [ :value |
				arguments at: index put: value.
				lastBlock value: arguments
			]
		]
	] ifFalse: [
		lastBlock
	]! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 13:31'!
promise
	^ self! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:14'!
resolveWith: arg
	"Resolve this promise"
	mutex critical: [
		isResolved ifTrue: [ ^ self ].
		value := arg.
		isResolved := true.
		resolvers ifNotNil: [resolvers do: [:r | self evaluateResolver: r]].
	]! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:00'!
value
	^ self checkForExceptionOrReturn: super wait! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:00'!
wait
	^ self checkForExceptionOrReturn: super wait! !

Morph subclass: #PPArrow
	instanceVariableNames: 'from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPArrow class methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 14:09'!
from: from to: to
	^ super new
		from: from;
		to: to;
		bounds: (from rect: to)
		yourself! !

!PPArrow class methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:07'!
new
	^ self shouldNotImplement! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:00'!
arrowBoundsAt: endPoint from: priorPoint 
	"Answer a triangle oriented along the line from priorPoint to endPoint."
	| d v angle wingBase arrowSpec length width |
	v := endPoint - priorPoint.
	angle := v degrees.
	d := 2.
	arrowSpec := self valueOfProperty: #arrowSpec ifAbsent: [5@4].
	length := arrowSpec x abs.  width := arrowSpec y abs.
	wingBase := endPoint + (Point r: d * length degrees: angle + 180.0).
	arrowSpec x >= 0
		ifTrue: [^ {	endPoint.
					wingBase + (Point r: d * width degrees: angle + 125.0).
					wingBase + (Point r: d * width degrees: angle - 125.0) }]
		ifFalse: ["Negative length means concave base."
				^ {	endPoint.
					wingBase + (Point r: d * width degrees: angle + 125.0).
					wingBase.
					wingBase + (Point r: d * width degrees: angle - 125.0) }]! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:06'!
drawArrowOn: aCanvas
	aCanvas
		line: from to: to width: 2 color: Color black;
		drawPolygon: (self arrowBoundsAt: to from: from) fillStyle: Color black! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:01'!
drawOn: aCanvas
	aCanvas
		translateBy: self owner position
		during: [ :canvas | self drawArrowOn: canvas ]! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:05'!
from
	^ from! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:05'!
from: aPoint
	from := aPoint! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:06'!
to
	^ to! !

!PPArrow methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 11:06'!
to: aPoint
	to := aPoint! !

Morph subclass: #PPMorph
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPMorph class methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:32'!
for: aModel
	^ super basicNew
		model: aModel;
		initialize;
		yourself! !

!PPMorph class methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:33'!
new
	^ self shouldNotImplement! !

!PPMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 16:23'!
model
	^ model! !

!PPMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 16:24'!
model: anObject
	model := anObject! !

PPMorph subclass: #PPPromiseMorph
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPPromiseMorph class methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 17:47'!
height
	^ 120! !

!PPPromiseMorph class methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 17:47'!
width
	^ 120! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 18:40'!
drawOn: aCanvas
	! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 14:53'!
handlesMouseDown: event
	self owner model selectPromise: model element.
	^ true! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:06'!
initialize
	| size |
	super initialize.
	
	size := self class width @ self class height.
	self addMorph: (content := BorderedMorph new).
	self extent: size.
	content extent: size.
	content layoutPolicy: TableLayout new.
	content addMorph: (TextMorph new contents: model element hash asString; hResizing: #spaceFill; vResizing: #shrinkWrap; yourself).
	content addMorph: (TextMorph new contents: model element class asString; hResizing: #spaceFill; vResizing: #shrinkWrap; yourself).
	self updatePromise! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:14'!
representsPromise: aPromise
	^ model element = aPromise! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:51'!
setSelected
	content color: (content color alpha: 0.7).
	self changed.! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:51'!
unsetSelected
	content color: (content color alpha: 1.0).
	content changed.! !

!PPPromiseMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:06'!
updatePromise
	| promise |
	promise := model element.
	promise isBroken ifTrue: [
		content color: Color lightRed
	] ifFalse: [
		promise isResolved ifTrue: [
			content color: Color lightGreen
		] ifFalse: [
			content color: Color lightGray
		]
	].
	content changed! !

PPMorph subclass: #PPVisualizationMorph
	instanceVariableNames: 'layouter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPVisualizationMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 16:20'!
drawOn: aCanvas
	! !

!PPVisualizationMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:33'!
initialize
	super initialize.
	
	layouter := PPHierarchicalLayouter new
		hierarchy: model hierarchy;
		yourself! !

!PPVisualizationMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 15:31'!
updateHierarchy
	layouter updateHierarchy.
	layouter layoutOn: self.! !

!PPVisualizationMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:57'!
updatePromises
	self submorphsDo: [ :morph |
		morph model ifNotNil: [ morph updatePromise ]
	]! !

!PPVisualizationMorph methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:15'!
updateSelection
	| selectedPromise |
	selectedPromise := model selectedPromise ifNil: [ ^ self ].
	self submorphsDo: [ :morph |
		morph model ifNotNil: [
			(morph representsPromise: selectedPromise) ifTrue: [
				morph setSelected
			] ifFalse: [
				morph unsetSelected
			]
		]
	]! !

TestCase subclass: #PPPromiseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Tests'!

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 12/14/2011 11:15'!
testDirectValue
	| promise result |
	promise := PPPromise new.
	result := Object new.
	
	self assert: [ promise directValue isNil ].
	
	promise resolveWith: result.
	
	self assert: [ promise directValue == result ].! !

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 12/14/2011 11:34'!
testPipelining
	| promise pipelinedPromise |
	promise := PPPromise new.
	pipelinedPromise := promise + 5.
	
	promise resolveWith: 5.
	self assert: [ pipelinedPromise value = 10 ].! !

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:16'!
testResolvingThrowsNoException
	| promise resolveValue |
	promise := PPPromise new.
	resolveValue := 5.
	promise resolveWith: resolveValue.
	self assert: promise value = resolveValue.
	promise resolveWith: 2.
	self assert: promise value = resolveValue! !

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 12/14/2011 11:30'!
testValue
	| promise hasWaited result process innerComparison innerHasWaited |
	promise := PPPromise new.
	hasWaited := false.
	result := Object new.
	
	process := [
		innerComparison := promise value == result.
		innerHasWaited := hasWaited.
	] fork.
	hasWaited := true.
	
	promise resolveWith: result.
	
	[ process isTerminated ] whileFalse.

	self assert: innerComparison.
	self assert: innerHasWaited.! !

!Object methodsFor: '*PromisePipelining' stamp: 'WS 1/13/2012 18:55'!
isExceptionReference
	^ false! !

!Object methodsFor: '*PromisePipelining' stamp: 'WS 1/11/2012 11:32'!
isPromise
	^ false! !

!Object methodsFor: '*PromisePipelining' stamp: 'WS 1/11/2012 13:30'!
promise
	^ PPObject for: self! !

Object subclass: #PPExceptionReference
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining'!

!PPExceptionReference class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 13:10'!
for: anException
	^ super new
		exception: anException;
		yourself! !

!PPExceptionReference methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 13:35'!
doesNotUnderstand: aMessage
	exception signal! !

!PPExceptionReference methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 13:10'!
exception: anException
	exception := anException! !

!PPExceptionReference methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 18:55'!
isExceptionReference
	^ true! !

!PPExceptionReference methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:15'!
raise
	exception signal! !

Object subclass: #PPFuture
	instanceVariableNames: 'promise'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining'!

!PPFuture class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:26'!
for: aPromise
	^ super new
		promise: aPromise;
		yourself! !

!PPFuture class methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:26'!
new
	^ self shouldNotImplement! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:28'!
directValue
	^ promise directValue! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 14:20'!
doesNotUnderstand: aMessage
	^ PPPromise promiseFrom: promise forMessage: aMessage onResolveSendTo: [ self directValue ]! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/13/2012 19:02'!
isBroken
	^ promise isBroken! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 12:11'!
isPromise
	^ true! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:10'!
isResolved
	^ promise isResolved! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:37'!
promise
	^ self! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:25'!
promise: aPromise
	promise := aPromise! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 11:26'!
value
	^ promise value! !

!PPFuture methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 12:04'!
whenResolved: aBlock
	^ promise whenResolved: aBlock! !

Object subclass: #PPHierarchicalLayouter
	instanceVariableNames: 'hierarchy sequences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPHierarchicalLayouter methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:28'!
hierarchy
	^ hierarchy! !

!PPHierarchicalLayouter methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:28'!
hierarchy: aHierarchy
	hierarchy := aHierarchy! !

!PPHierarchicalLayouter methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:50'!
initialize
	super initialize.
	
	self invalidateCache! !

!PPHierarchicalLayouter methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:50'!
invalidateCache
	sequences := OrderedCollection new! !

!PPHierarchicalLayouter methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:47'!
layoutOn: aMorph
	| elementToMorphs promiseToMorphs topLeft horizontalPadding verticalPadding |
	topLeft := 20 @ 20.
	horizontalPadding := 20.
	verticalPadding := 50.
	elementToMorphs := Dictionary new.
	promiseToMorphs := Dictionary new.
	aMorph removeAllMorphs.
	sequences withIndexDo: [ :sequence :sequenceIndex |
		sequence withIndexDo: [ :hierarchyElement :elementIndex | | morph |
			morph := elementToMorphs at: hierarchyElement put: (PPPromiseMorph for: hierarchyElement).
			aMorph addMorph: morph.
			morph position: topLeft + ((PPPromiseMorph width + horizontalPadding * (elementIndex-1)) @ (PPPromiseMorph height + verticalPadding * (sequenceIndex-1)))
		]
	].
	hierarchy childrenDo: [ :hierarchyElement |
		hierarchyElement children do: [ :association |
			aMorph addMorphBack: (PPArrow
				from: (elementToMorphs at: hierarchyElement) bounds bottomCenter
				to: (elementToMorphs at: association value) bounds topCenter
			)
		]
	].
	elementToMorphs keysAndValuesDo: [ :element :morph | promiseToMorphs at: element element put: morph ].
	aMorph model registerMorphs: promiseToMorphs.
	aMorph bounds: aMorph boundingBoxOfSubmorphs! !

!PPHierarchicalLayouter methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 16:04'!
updateHierarchy
	| insertedElements |
	self invalidateCache.
	
	insertedElements := Set new.
	hierarchy childrenDo: [ :child |
		(insertedElements includes: child) ifFalse: [
			(sequences at: child hierarchyDepth ifAbsentPut: [ OrderedCollection new ])
				add: child.
			insertedElements add: child
		]
	]! !

Object subclass: #PPHierarchyElement
	instanceVariableNames: 'element children parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:35'!
addChild: anObject withRelation: anotherObject
	children add: anotherObject -> anObject.
	anObject parent: self! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:20'!
asOrderedCollection
	! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:26'!
children
	^ children! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 15:49'!
childrenDo: aBlock
	children do: [ :child |
		aBlock value: child value.
		child value childrenDo: aBlock
	]! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:36'!
disconnectFromParent
	parent ifNil: [ ^ self ].
	parent parent ifNil: [ parent removeChildren: self ]! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:25'!
element
	^ element! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:25'!
element: anObject
	element := anObject! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:53'!
empty
	element := nil.
	parent := nil.
	children := OrderedCollection new! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:19'!
hierarchyDepth
	^ parent ifNil: [ 0 ] ifNotNil: [ parent hierarchyDepth + 1 ]! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:26'!
initialize
	super initialize.
	
	children := OrderedCollection new! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:25'!
parent
	^ parent! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:36'!
parent: anObject
	(parent isNil or: [ parent hierarchyDepth < anObject hierarchyDepth ]) ifTrue: [
		self disconnectFromParent.
		parent := anObject
	]! !

!PPHierarchyElement methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:39'!
removeChildren: anObject
	children removeAllSuchThat: [ :assoc | assoc value = anObject ]! !

Object subclass: #PPObject
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining'!

!PPObject class methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 13:30'!
for: anObject
	^ super new
		object: anObject;
		yourself! !

!PPObject class methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 13:30'!
new
	^ self shouldNotImplement! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 16:00'!
directValue
	^ object! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:23'!
doesNotUnderstand: aMessage
	^ PPPromise promiseFrom: self forMessage: aMessage onResolveSendTo: [ object ]! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:23'!
isBroken
	^ false! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:24'!
isResolved
	^ true! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 13:29'!
object
	^ object! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 13:29'!
object: anObject
	object := anObject! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/11/2012 13:30'!
promise
	^ self! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 16:00'!
value
	^ object! !

!PPObject methodsFor: 'as yet unclassified' stamp: 'WS 1/12/2012 12:03'!
whenResolved: aBlock
	^ aBlock cull: object! !

Object subclass: #PPPromisePipeliningVisualisation
	instanceVariableNames: 'window codeWidget scrollPane executeButton listWidget canvasWidget resolveButton resolveText promises selectedPromise promiseToMorphs elementToHierarchy hierarchy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPPromisePipeliningVisualisation class methodsFor: 'as yet unclassified' stamp: 'WS 12/20/2011 21:56'!
open
	^ self new
		open;
		yourself! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:20'!
accept: aString
	^ false! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:20'!
buttonState
	^ true! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:16'!
codeText
	^ '| p1 p2|
p1 := PPPromise new.
p2 := PPPromise new.
p1 + p2.
'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:20'!
defaultExtent
	^ 1200 @ 800! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:20'!
derive: subPromise from: parentPromise withMessage: aMessage
	(self hierarchyElementFor: parentPromise)
		addChild: (self hierarchyElementFor: subPromise)
		withRelation: aMessage.
	
	aMessage arguments do: [ :argument |
		argument isPromise ifTrue: [
			(self hierarchyElementFor: argument)
				addChild: (self hierarchyElementFor: subPromise)
				withRelation: aMessage.
		]
	]! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:02'!
doItContext
	^ nil! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:02'!
doItReceiver
	^ self! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:20'!
executeButtonText
	^ 'Execute'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 14:44'!
executeCode
	| creationSignalId connectSignalId |
	creationSignalId := PPPromise onCreation: [ :promise | self promiseCreated: promise ].
	connectSignalId := PPPromise onConnect: [ :referenced :promise :message | self derive: promise from: referenced withMessage: message ].
	self initializePromises.
	listWidget updateList.
	codeWidget textMorph editor
		selectAll;
		evaluateSelection.
	PPPromise disconnectCreation: creationSignalId.
	PPPromise disconnectConnect: connectSignalId.
	canvasWidget updateHierarchy.
	scrollPane setScrollDeltas! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 13:35'!
hierarchy
	^ hierarchy! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:17'!
hierarchyElementFor: anObject
	^ elementToHierarchy at: anObject ifAbsentPut: [ | element |
		element := PPHierarchyElement new
			element: anObject;
			parent: hierarchy;
			yourself.
		hierarchy addChild: element withRelation: nil.
		element
	]! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:02'!
initialize
	super initialize.
	self initializePromises.
	
	codeWidget := PluggableTextMorphPlus on: self text: #codeText accept: #accept:.
	codeWidget styler: (SHTextStylerST80 new view: codeWidget; yourself).
	codeWidget wantsFrameAdornments: false.
	codeWidget textMorph editor model: self.
	
	executeButton := PluggableButtonMorphPlus on: self getState: #buttonState action: #executeCode label: #executeButtonText.
	
	listWidget := PluggableListMorph on: self list: #promises selected: #selectedPromiseIndex changeSelected: #selectedPromiseIndex:.
	
	scrollPane := ScrollPane new.
	canvasWidget := PPVisualizationMorph for: self.
	scrollPane scroller addMorph: canvasWidget.
	
	resolveButton := PluggableButtonMorphPlus on: self getState: #buttonState action: #resolveSelectedPromise label: #resolveButtonText.
	resolveText := PluggableTextMorphPlus on: self text: #resolveText accept: #accept:.
	resolveText styler: (SHTextStylerST80 new view: resolveText; yourself).
	resolveText wantsFrameAdornments: false.
	resolveText textMorph editor model: self.
	
	window := SystemWindow new.
	window setWindowColor: window defaultColor.
	window setLabel: self labelString.
	window addMorph: codeWidget frame: (0 @ 0 corner: 0.7 @ 0.3).
	window addMorph: executeButton frame: (0.7 @ 0 corner: 1 @ 0.05).
	window addMorph: listWidget frame: (0.7 @ 0.05 corner: 1 @ 0.75).
	window addMorph: scrollPane frame: (0 @ 0.3 corner: 0.7 @ 1).
	window addMorph: resolveButton frame: (0.7 @ 0.75 corner: 1 @ 0.8).
	window addMorph: resolveText frame: (0.7 @ 0.8 corner: 1 @ 1).
	window model: self.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 16:24'!
initializePromises
	selectedPromise := 0.
	promises := OrderedCollection new.
	promiseToMorphs := Dictionary new.
	hierarchy ifNil: [ hierarchy:= PPHierarchyElement new ] ifNotNil: [ hierarchy empty ].
	elementToHierarchy := Dictionary new.
	canvasWidget ifNotNil: [ canvasWidget updateHierarchy ]! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:20'!
labelString
	^ 'Promise Pipelining Visualisation'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:20'!
nextPromiseId
	| nextId |
	nextId := nextPromiseId.
	nextPromiseId := nextPromiseId + 1.
	^ nextId! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:19'!
open
	window openInWorldExtent: self defaultExtent! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 16:22'!
promiseCreated: promise
	(promises includes: promise) ifTrue: [ ^ self ].
	promises add: promise.
	promise whenResolved: [ self resolved: promise ].
	listWidget updateList.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:49'!
promises
	^ promises collect: [ :promise | 'Promise ', (promise isResolved ifTrue: [
			promise isBroken ifTrue: [ 'is broken ' ] ifFalse: [ 'resolved with ' ,promise value ] ] ifFalse: [ 'not resolved' ]) ]! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 16:26'!
registerMorphs: aDictionary
	promiseToMorphs := aDictionary! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:19'!
resolveButtonText
	^ 'Resolve With'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 18:09'!
resolveSelectedPromise
	self resolveSelectedPromiseWith: (resolveText textMorph editor evaluateSelection)! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:53'!
resolveSelectedPromiseWith: anObject
	| promise |
	promise := self selectedPromise ifNil: [ ^ self ].
	
	promise resolveWith: anObject.
	resolveText update: #resolveText.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/6/2012 17:19'!
resolveText
	^ ''! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:57'!
resolved: promise
	listWidget updateList.
	canvasWidget updatePromises.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:19'!
selectPromise: promise
	selectedPromise := promises indexOf: promise.
	canvasWidget updateSelection.
	listWidget updateList! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/16/2012 14:04'!
selectedClass
	^ nil! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:03'!
selectedPromise
	^ promises at: selectedPromise ifAbsent: [ nil ]! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:02'!
selectedPromiseIndex
	^ selectedPromise! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/17/2012 15:02'!
selectedPromiseIndex: newIndex
	selectedPromise := newIndex.
	canvasWidget updateSelection.
	listWidget updateList! !

Object subclass: #PPSignal
	instanceVariableNames: 'callbacks nextId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Signals'!

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:45'!
connect: aBlock
	| id |
	id := nextId.
	callbacks at: id put: aBlock.
	nextId := nextId + 1.
	^ id
	! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:45'!
disconnect: anId
	^ callbacks removeKey: anId ifAbsent: []! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:40'!
disconnectAll
	^ callbacks removeAll! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:34'!
emit
	self emitWithArguments: #()! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:36'!
emitWith: arg1
	self emitWithArguments: (Array with: arg1)! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:36'!
emitWith: arg1 with: arg2
	self emitWithArguments: (Array with: arg1 with: arg2)! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:36'!
emitWith: arg1 with: arg2 with: arg3
	self emitWithArguments: (Array with: arg1 with: arg2 with: arg3)! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:57'!
emitWithArguments: arguments
	callbacks do: [ :callback | 
		callback valueWithArguments: (self resizeArguments: arguments toFit: callback argumentCount)
	]! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:28'!
initialize
	super initialize.
	callbacks := Dictionary new.
	nextId := 0.! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:40'!
resizeArguments: arguments toFit: aSize
	| newArgs |
	newArgs := Array new: aSize.
	1 to: aSize do: [ :i |
		newArgs at: i put: (arguments at: i ifAbsent: [ nil ])
	].
	^ newArgs! !

Object subclass: #PPSignalHandler
	instanceVariableNames: 'signals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Signals'!

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'WS 1/4/2012 13:41'!
connect: signalName to: aBlock
	^ ( signals at: signalName ifAbsentPut: [ PPSignal new ])
		connect: aBlock! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:46'!
disconnect: signalName from: anId
	(signals at: signalName ifAbsent: [ ^ self ]) disconnect: anId! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:47'!
emit: signalName
	 (signals at: signalName ifAbsentPut: [ ^ self ]) emit! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:05'!
emit: signalName with: arg1
	 (signals at: signalName ifAbsentPut: [ ^ self ])
		emitWithArguments: (Array with: arg1)! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:05'!
emit: signalName with: arg1 with: arg2
	 (signals at: signalName ifAbsentPut: [ ^ self ])
		emitWithArguments: (Array with: arg1 with: arg2)! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:05'!
emit: signalName with: arg1 with: arg2 with: arg3
	 (signals at: signalName ifAbsentPut: [ ^ self ])
		emitWithArguments: (Array with: arg1 with: arg2 with: arg3)! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:47'!
emit: signalName withArguments: arguments
	 (signals at: signalName ifAbsentPut: [ ^ self ]) emitWithArguments: arguments! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:36'!
initialize
	super initialize.
	signals := Dictionary new! !
